local FOLDER_NAME, private = ...
local _G = _G
local LS = _G.LibStub
local core = LS("AceAddon-3.0"):NewAddon(FOLDER_NAME, "AceEvent-3.0", "AceHook-3.0")
local L = LS("AceLocale-3.0"):GetLocale(FOLDER_NAME)
local LSM = LS("LibSharedMedia-3.0")
LSM:Register("font", "Komika Text Kaps", [[Interface\AddOns\Broker_ProfessionMats\media\KOMTXTK_.ttf]])
local GBS = LS("LibGuildBankScan")
local version = GetAddOnMetadata(FOLDER_NAME, "X-Version")
local author = GetAddOnMetadata(FOLDER_NAME, "X-Author")
private.version = version
private.author = author
local strformat, pairs, strupper, select, type, strgmatch, match, sub = string.format, pairs, strupper, select, type, string.gmatch, match, sub
local UnitName, UnitClass, GetItemInfo, GetItemCount, IsInGuild, ReloadUI = UnitName, UnitClass, GetItemInfo, GetItemCount, IsInGuild, ReloadUI
local playerName, playerClass = UnitName("player"), strupper(select(2, UnitClass("player")))
local charsList, player, tempChar, firstrun, BPMcache, entries  = {}, nil, nil, nil, nil, 0
local font1 = CreateFont("BPMNameHeader")
local font2 = CreateFont("BPMColHeader")
local font3 = CreateFont("BPMText")
local font4 = CreateFont("BPMNumbers")
local font5 = CreateFont("BPMTotHeader")

StaticPopupDialogs["BPMReload"] = {
	text = "|cffff0000"..L["Disable and Reload the UI."].."|r",
	button1 = L["Now"],
	button2 = L["Later"],
	OnAccept = function(self)
		ReloadUI()
	end,
	whileDead = 1,
	timeout = 0,
	hideOnEscape = 0
}

local defaults = {
	profile = {
		modules = {
			['*'] = false,
		},
		firstrun = false,
		showShort = false,
		showIcons = true,
		iconsOnly = false,
		trackerColor = true,
		tooltipShowShort = false,
		tooltipIcons = true,
		tooltipIconsOnly = false,
		lineColor = true,
		textColor = true,
		showZero = false,
		showBank = true,
		showAlts = true,
		showGuild = false,
		showTotal = true,
		scale = 1,
		scroll = 900,
		showallopts = false,
		showxps = false,
		namefont = "Komika Text Kaps",
		namefontsize = 20,
		namefontol = "None",
		headerfont = "Komika Text Kaps",
		headerfontsize = 17,
		headerfontol = "None",
		textfont = "Komika Text Kaps",
		textfontsize = 15,
		textfontol = "None",
		numberfont = "Komika Text Kaps",
		numberfontsize = 17,
		numberfontol = "None",
	},
	realm = {
		chars = {
			['*'] = {
				data = {
					['*'] = { inv = 0, bank = 0, guild = 0 },
				},
			}
		}
	}
}

function core:updateFonts()
	font1:SetFont(LSM:Fetch("font", core.db.profile.namefont), core.db.profile.namefontsize, core.db.profile.namefontol)
	font2:SetFont(LSM:Fetch("font", core.db.profile.headerfont), core.db.profile.headerfontsize, core.db.profile.headerfontol)
	font2:SetTextColor(1, .85, 0)
	font3:SetFont(LSM:Fetch("font", core.db.profile.textfont), core.db.profile.textfontsize, core.db.profile.textfontol)
	font4:SetFont(LSM:Fetch("font", core.db.profile.numberfont), core.db.profile.numberfontsize, core.db.profile.numberfontol)
	font5:SetFont(LSM:Fetch("font", core.db.profile.namefont), core.db.profile.namefontsize, core.db.profile.namefontol)
	font5:SetTextColor(1, .85, 0)
end

function core:bagUpdate()
	for k,v in self:IterateModules() do
		if v:IsEnabled() then
			v:count()
		end
	end
end

function core:guildCount()
	for k,v in self:IterateModules() do
		if v:IsEnabled() then
			v:countGuild()
		end
	end
	private.scanPending = false
end

function core:guildScan()
	if not IsInGuild() then return end
	private.scanPending = true
	for k,v in self:IterateModules() do
		if v:IsEnabled() then
			core:textUpdate(k)
		end
	end
end

function core:OnInitialize()
	self.db = LS("AceDB-3.0"):New("Broker_ProfessionMatsDB", defaults, true)
	self.db.RegisterCallback(self, "OnProfileChanged", "refresh")
	self.db.RegisterCallback(self, "OnProfileCopied", "refresh")
	self.db.RegisterCallback(self, "OnProfileReset", "profileReset")
	player = self.db.realm.chars[playerName]
	private.player = player
	if not player.class then
		player.class = playerClass
	end
	for name in pairs(self.db.realm.chars) do
		charsList[name] = name
	end
	if not self.db.profile.firstrun then
		core:setModuleEnabled("Elements", true)
		self.db.profile.firstrun = true
	end
	private.charsList = charsList
	core:updateFonts()
	self:setupOptions()
end

function core:OnEvent(event, ...)
	local itemid = ...
	if event == "GET_ITEM_INFO_RECEIVED" then
		BPMcache[itemid] = GetItemInfo(itemid)
	end
end

function core:OnEnable()
	core:RegisterEvent("GET_ITEM_INFO_RECEIVED", "OnEvent")
	core:RegisterEvent("BAG_UPDATE_DELAYED", "bagUpdate")
	if IsInGuild() then
		core:RegisterEvent("GUILDBANKFRAME_OPENED", "guildScan")
		core:RegisterEvent("GUILDBANKFRAME_CLOSED", "guildCount")
	end
end

function core:profileReset()
	ReloadUI()
end

function core:refresh()
	for k,v in self:IterateModules() do
		if self:getModuleEnabled(k) and not v:IsEnabled() then
			self:EnableModule(k)
		elseif not self:getModuleEnabled(k) and v:IsEnabled() then
			self:DisableModule(k)
		end
	end
end

function core:applySettings()
	for k,v in self:IterateModules() do
		if self:getModuleEnabled(k) and v:IsEnabled() then
			v.refresh()
		end
	end
end

function core:getNamespace(MODNAME)
	return self.db:GetNamespace(MODNAME)
end

function core:getModuleEnabled(module)
	return core.db.profile.modules[module]
end

function core:setModuleEnabled(module, value)
	local old = core.db.profile.modules[module]
	core.db.profile.modules[module] = value
	if old ~= value then
		if value then
			self:EnableModule(module)
		else
			self:DisableModule(module)
		end
	end
end

function core:itemName(itemid)
	local name = GetItemInfo(itemid) or BPMcache[itemid]
	return name
end

function core:shortText(text)
	local stext = ""
	for word in strgmatch(text, '%S+%s?') do
		local first = (word:match('%w') or '' ):sub(1, 1)
		local second = word:match('[^%s%w]') or ''
		stext = stext .. first .. second
	end
	return stext
end

function core:saveCounts(itemid)
	local name = GetItemInfo(itemid)
	if not name then
		if BPMcache == nil then
			BPMcache = {}
		end
		BPMcache[itemid] = "???"
		entries = entries + 1
	end
	if entries == 0 then
		core:UnregisterEvent("GET_ITEM_INFO_RECEIVED")
		BPMcache = nil
	end
	local inv = GetItemCount(itemid)
	local bank = GetItemCount(itemid, true)
	bank = bank - inv
	player.data[itemid].inv = inv or 0
	player.data[itemid].bank = bank or 0
end

function core:saveGuildCounts(itemid)
	local total = GBS:scanResult(itemid)
	player.data[itemid].guild = total or 0
end

function core:charHasElement(itemid)
	if player.data[itemid].inv > 0 or (self.db.profile.showBank and player.data[itemid].bank > 0) or (self.db.profile.showGuild and player.data[itemid].guild > 0) then
		return true
	end
	if self.db.profile.showAlts then
		for char, _ in pairs(self.db.realm.chars) do
			if self.db.realm.chars[char].data[itemid].inv > 0 or (self.db.profile.showBank and self.db.realm.chars[char].data[itemid].bank > 0) or (self.db.profile.showGuild and self.db.realm.chars[char].data[itemid].guild > 0) then
				return true
			end
		end
	end
	return false
end

function core:invCountString(itemid, char)
	if char then
		tempChar = self.db.realm.chars[char]
	else
		tempChar = player
	end
	return self:color(tempChar.data[itemid].inv)
end

function core:bankCountString(itemid, char)
	if char then
		tempChar = self.db.realm.chars[char]
	else
		tempChar = player
	end
	return self:color(tempChar.data[itemid].bank)
end

function core:guildCountString(itemid, char)
	if char then
		tempChar = self.db.realm.chars[char]
	else
		tempChar = player
	end
	return self:color(tempChar.data[itemid].guild)
end

function core:color(count)
	local color
	if core.db.profile.textColor then
		local perc = 0
		local max = 200
		if max ~= 0 then
			perc = count / max
		end
		local r1, g1, b1
		local r2, g2, b2
		if perc <= 0.5 then
			perc = perc * 2
			r1, g1, b1 = 1, 0, 0
			r2, g2, b2 = 1, 1, 0
		else
			perc = perc * 2 - 1
			r1, g1, b1 = 1, 1, 0
			r2, g2, b2 = 0, 1, 0
		end
		local r, g, b = r1 + (r2 - r1) * perc, g1 + (g2 - g1) * perc, b1 + (b2 - b1) * perc
		if r < 0 then
			r = 0
		elseif r > 1 then
			r = 1
		end
		if g < 0 then
			g = 0
		elseif g > 1 then
			g = 1
		end
		if b < 0 then
			b = 0
		elseif b > 1 then
			b = 1
		end
		r, g, b = r * 255, g * 255, b * 255
		color = strformat("%02x%02x%02x", r, g, b)
	else
		color = "ffffff"
	end
	count = ("|cff"..color..count.."|r")
	return count
end
