local FOLDER_NAME, private = ...
local _G = _G
local LS = _G.LibStub
local core = LS("AceAddon-3.0"):GetAddon(FOLDER_NAME)
local L = LS("AceLocale-3.0"):GetLocale(FOLDER_NAME)
local wipe, pairs, strsub, pairs = wipe, pairs, strsub, pairs
local CreateFrame, UIDropDownMenu_AddButton, CloseDropDownMenus, ToggleDropDownMenu, HideUIPanel = CreateFrame, UIDropDownMenu_AddButton, CloseDropDownMenus, ToggleDropDownMenu, HideUIPanel
local xp

function core:getDropdown(MODNAME)
	if MODNAME == "Cloth" then return BPMClothDD
		elseif MODNAME == "Cooking" then return BPMCookingDD
		elseif MODNAME == "Elements" then return BPMElementsDD
		elseif MODNAME == "Enchanting" then return BPMEnchantingDD
		elseif MODNAME == "Gems" then return BPMGemsDD
		elseif MODNAME == "Herbs" then return BPMHerbsDD
		elseif MODNAME == "Inscription" then return BPMInscriptionDD
		elseif MODNAME == "Mining" then return BPMMiningDD
		elseif MODNAME == "Skinning" then return BPMSkinningDD
	end
end

function core:closeDropdown()
	CloseDropDownMenus()
end

function core:itemString(MODNAME, xp, k, itemid)
	local color, name, item
	name = core:itemName(itemid)
	if core.db.profile.textColor then
		color = private.data[MODNAME][xp].color[k]
	else
		color = "FFFFFF"
	end
	item = "|cff"..color..name.."|r"
	return item
end

function core:createDropdownMenu(MODNAME, dropdownname)
	local namespace = core:getNamespace(MODNAME)
	local dropdownmenu = CreateFrame("Frame", dropdownname)
	dropdownmenu.displayMode = "MENU"
	dropdownmenu.info = {}
	dropdownmenu.uncheck = function(dropdownbutton) _G[dropdownbutton:GetName() .. "Check"]:Hide() end
	local info = {}
	dropdownmenu.initialize = function(self, level)
		if _G.InterfaceOptionsFrame:IsShown() then
			HideUIPanel(_G.InterfaceOptionsFrame)
		end
		if not level then return end
		local info = self.info
		wipe(info)
		if level == 1 then
			info.keepShownOnClick = 1
			info.isTitle = 1
			info.text = "BPM-"..MODNAME
			info.notCheckable = 1
			UIDropDownMenu_AddButton(info, level)
			info.isTitle = nil
			info.text = ""
			info.disabled = 1
			UIDropDownMenu_AddButton(info, level)
			info.isTitle = 1
			info.text = L["Tracker Options"]
			UIDropDownMenu_AddButton(info, level)
			info.isTitle = nil
			info.disabled = nil
			info.hasArrow = 1
			if private.data[MODNAME].van then
				info.text = ("|cffffff99" .. L["Vanilla"] .. "|r")
				info.value = "submenu1"
				info.func = self.uncheck
				UIDropDownMenu_AddButton(info, level)
			end
			info.text = ("|cff00ff00" .. L["The Burning Crusade"] .. "|r")
			info.value = "submenu2"
			info.func = self.uncheck
			UIDropDownMenu_AddButton(info, level)
			info.text = ("|cff0066ff" .. L["Wrath of the Lich King"] .. "|r")
			info.value = "submenu3"
			info.func = self.uncheck
			UIDropDownMenu_AddButton(info, level)
			info.text = ("|cffff6600" .. L["Cataclysm"] .. "|r")
			info.value = "submenu4"
			info.func = self.uncheck
			UIDropDownMenu_AddButton(info, level)
			info.text = ("|cff19ac87" .. L["Mists of Pandaria"] .. "|r")
			info.value = "submenu5"
			info.func = self.uncheck
			UIDropDownMenu_AddButton(info, level)
			info.text = ("|cfffd482f" .. L["Warlords of Draenor"] .. "|r")
			info.value = "submenu6"
			info.func = self.uncheck
			UIDropDownMenu_AddButton(info, level)
			info.text = ("|cff299f06" .. L["Legion"] .. "|r")
			info.value = "submenu7"
			info.func = self.uncheck
			UIDropDownMenu_AddButton(info, level)
			info.hasArrow = nil
			if core.db.profile.showallopts then
				info.notCheckable = nil
				info.text = L["Show icons on the tracker."]
				info.func = function() core.db.profile.showIcons = not core.db.profile.showIcons core:closeDropdown() core:applySettings() ToggleDropDownMenu(1, nil, private.dropdown, private.fname, 0, 0) end
				info.checked = function() return core.db.profile.showIcons end
				UIDropDownMenu_AddButton(info, level)
				if core.db.profile.showIcons then
					info.text = L["Show icons only on the tracker."]
					info.func = function() core.db.profile.iconsOnly = not core.db.profile.iconsOnly core:closeDropdown() core:applySettings() ToggleDropDownMenu(1, nil, private.dropdown, private.fname, 0, 0) end
					info.checked = function() return core.db.profile.iconsOnly end
					UIDropDownMenu_AddButton(info, level)
				end
				if not core.db.profile.iconsOnly or not core.db.profile.showIcons then
					info.text = L["Show short text on the tracker."]
					info.func = function() core.db.profile.showShort = not core.db.profile.showShort core:applySettings() end
					info.checked = function() return core.db.profile.showShort end
					UIDropDownMenu_AddButton(info, level)
				end
				info.text = L["Color text in the tracker."]
				info.func = function() core.db.profile.trackerColor = not core.db.profile.trackerColor core:applySettings() end
				info.checked = function() return core.db.profile.trackerColor end
				UIDropDownMenu_AddButton(info, level)
			end
			if core.db.profile.showxps then
				info.notCheckable = 1
				info.text = ""
				info.disabled = 1
				UIDropDownMenu_AddButton(info, level)
				info.isTitle = 1
				info.text = L["Tooltip Options"]
				UIDropDownMenu_AddButton(info, level)
				info.isTitle = nil
				info.notCheckable = nil
				info.disabled = nil
				info.text = ("|cffffff99" .. L["Vanilla"] .. "|r")
				info.func = function() namespace.profiles.Default.van = not namespace.profiles.Default.van end
				info.checked = function() return namespace.profiles.Default.van end
				UIDropDownMenu_AddButton(info, level)
				info.text = ("|cff00ff00" .. L["The Burning Crusade"] .. "|r")
				info.func = function() namespace.profiles.Default.bc = not namespace.profiles.Default.bc end
				info.checked = function() return namespace.profiles.Default.bc end
				UIDropDownMenu_AddButton(info, level)
				info.text = ("|cff0066ff" .. L["Wrath of the Lich King"] .. "|r")
				info.func = function() namespace.profiles.Default.lk = not namespace.profiles.Default.lk end
				info.checked = function() return namespace.profiles.Default.lk end
				UIDropDownMenu_AddButton(info, level)
				info.text = ("|cffff6600" .. L["Cataclysm"] .. "|r")
				info.func = function() namespace.profiles.Default.cata = not namespace.profiles.Default.cata end
				info.checked = function() return namespace.profiles.Default.cata end
				UIDropDownMenu_AddButton(info, level)
				info.text = ("|cff19ac87" .. L["Mists of Pandaria"] .. "|r")
				info.func = function() namespace.profiles.Default.mop = not namespace.profiles.Default.mop end
				info.checked = function() return namespace.profiles.Default.mop end
				UIDropDownMenu_AddButton(info, level)
				info.text = ("|cfffd482f" .. L["Warlords of Draenor"] .. "|r")
				info.func = function() namespace.profiles.Default.wod = not namespace.profiles.Default.wod end
				info.checked = function() return namespace.profiles.Default.wod end
				UIDropDownMenu_AddButton(info, level)
				info.text = ("|cff299f06" .. L["Legion"] .. "|r")
				info.func = function() namespace.profiles.Default.leg = not namespace.profiles.Default.leg end
				info.checked = function() return namespace.profiles.Default.leg end
				UIDropDownMenu_AddButton(info, level)
			end
			if not core.db.profile.showxps and core.db.profile.showallopts then
				info.notCheckable = 1
				info.text = ""
				info.disabled = 1
				UIDropDownMenu_AddButton(info, level)
				info.isTitle = 1
				info.text = L["Tooltip Options"]
				UIDropDownMenu_AddButton(info, level)
				info.isTitle = nil
				info.notCheckable = nil
				info.disabled = nil
			end
			if core.db.profile.showallopts then
				info.text = L["Show zero counts in the tooltip."]
				info.func = function() core.db.profile.showZero = not core.db.profile.showZero end
				info.checked = function() return core.db.profile.showZero end
				UIDropDownMenu_AddButton(info, level)
				info.text = L["Show bank counts in the tooltip."]
				info.func = function() core.db.profile.showBank = not core.db.profile.showBank end
				info.checked = function() return core.db.profile.showBank end
				UIDropDownMenu_AddButton(info, level)
				info.text = L["Show alt counts in the tooltip."]
				info.func = function() core.db.profile.showAlts = not core.db.profile.showAlts end
				info.checked = function() return core.db.profile.showAlts end
				UIDropDownMenu_AddButton(info, level)
				info.text = L["Show guild counts in the tooltip."]
				info.func = function() core.db.profile.showGuild = not core.db.profile.showGuild end
				info.checked = function() return core.db.profile.showGuild end
				UIDropDownMenu_AddButton(info, level)
				info.text = L["Show totals in the tooltip."]
				info.func = function() core.db.profile.showTotal = not core.db.profile.showTotal end
				info.checked = function() return core.db.profile.showTotal end
				UIDropDownMenu_AddButton(info, level)
				info.text = L["Show icons in the tooltip."]
				info.func = function() core.db.profile.tooltipIcons = not core.db.profile.tooltipIcons core:closeDropdown() ToggleDropDownMenu(1, nil, private.dropdown, private.fname, 0, 0) end
				info.checked = function() return core.db.profile.tooltipIcons end
				UIDropDownMenu_AddButton(info, level)
				if core.db.profile.tooltipIcons then
					info.text = L["Show icons only in the tooltip."]
					info.func = function() core.db.profile.tooltipIconsOnly = not core.db.profile.tooltipIconsOnly core:closeDropdown() ToggleDropDownMenu(1, nil, private.dropdown, private.fname, 0, 0) end
					info.checked = function() return core.db.profile.tooltipIconsOnly end
					UIDropDownMenu_AddButton(info, level)
				end
				if not core.db.profile.tooltipIconsOnly or not core.db.profile.tooltipIcons then
					info.text = L["Show short text in the tooltip."]
					info.func = function() core.db.profile.tooltipShowShort = not core.db.profile.tooltipShowShort end
					info.checked = function() return core.db.profile.tooltipShowShort end
					UIDropDownMenu_AddButton(info, level)
				end
				info.text = L["Color lines in the tooltip."]
				info.func = function() core.db.profile.lineColor = not core.db.profile.lineColor end
				info.checked = function() return core.db.profile.lineColor end
				UIDropDownMenu_AddButton(info, level)
				info.text = L["Color text in the tooltip."]
				info.func = function() core.db.profile.textColor = not core.db.profile.textColor end
				info.checked = function() return core.db.profile.textColor end
				UIDropDownMenu_AddButton(info, level)
			end
			info.notCheckable = 1
			info.text = ""
			info.disabled = 1
			UIDropDownMenu_AddButton(info, level)
			info.disabled = nil
			info.text = L["Close"]
			info.func = function() core:closeDropdown() end
			UIDropDownMenu_AddButton(info, level)
		elseif level == 2 then
			if UIDROPDOWNMENU_MENU_VALUE == "submenu1" then
				info.keepShownOnClick = 1
				xp = "van"
				for k, itemid in pairs(private.data[MODNAME].van.id) do
					local fixeditem = strsub(itemid, 1, -1)
					info.text = core:itemString(MODNAME, xp, k, itemid)
					info.func = function() namespace.profiles.Default.tracking[fixeditem] = not namespace.profiles.Default.tracking[fixeditem] core:textUpdate(MODNAME) end
					info.checked = function() return namespace.profiles.Default.tracking[fixeditem] end
					UIDropDownMenu_AddButton(info, level)
				end
			end
			if UIDROPDOWNMENU_MENU_VALUE == "submenu2" then
				info.keepShownOnClick = 1
				xp = "bc"
				for k, itemid in pairs(private.data[MODNAME].bc.id) do
					local fixeditem = strsub(itemid, 1, -1)
					info.text = core:itemString(MODNAME, xp, k, itemid)
					info.func = function() namespace.profiles.Default.tracking[fixeditem] = not namespace.profiles.Default.tracking[fixeditem] core:textUpdate(MODNAME) end
					info.checked = function() return namespace.profiles.Default.tracking[fixeditem] end
					UIDropDownMenu_AddButton(info, level)
				end
			end
			if UIDROPDOWNMENU_MENU_VALUE == "submenu3" then
				info.keepShownOnClick = 1
				xp = "lk"
				for k, itemid in pairs(private.data[MODNAME].lk.id) do
					local fixeditem = strsub(itemid, 1, -1)
					info.text = core:itemString(MODNAME, xp, k, itemid)
					info.func = function() namespace.profiles.Default.tracking[fixeditem] = not namespace.profiles.Default.tracking[fixeditem] core:textUpdate(MODNAME) end
					info.checked = function() return namespace.profiles.Default.tracking[fixeditem] end
					UIDropDownMenu_AddButton(info, level)
				end
			end
			if UIDROPDOWNMENU_MENU_VALUE == "submenu4" then
				info.keepShownOnClick = 1
				xp = "cata"
				for k, itemid in pairs(private.data[MODNAME].cata.id) do
					local fixeditem = strsub(itemid, 1, -1)
					info.text = core:itemString(MODNAME, xp, k, itemid)
					info.func = function() namespace.profiles.Default.tracking[fixeditem] = not namespace.profiles.Default.tracking[fixeditem] core:textUpdate(MODNAME) end
					info.checked = function() return namespace.profiles.Default.tracking[fixeditem] end
					UIDropDownMenu_AddButton(info, level)
				end
			end
			if UIDROPDOWNMENU_MENU_VALUE == "submenu5" then
				info.keepShownOnClick = 1
				xp = "mop"
				for k, itemid in pairs(private.data[MODNAME].mop.id) do
					local fixeditem = strsub(itemid, 1, -1)
					info.text = core:itemString(MODNAME, xp, k, itemid)
					info.func = function() namespace.profiles.Default.tracking[fixeditem] = not namespace.profiles.Default.tracking[fixeditem] core:textUpdate(MODNAME) end
					info.checked = function() return namespace.profiles.Default.tracking[fixeditem] end
					UIDropDownMenu_AddButton(info, level)
				end
			end
			if UIDROPDOWNMENU_MENU_VALUE == "submenu6" then
				info.keepShownOnClick = 1
				xp = "wod"
				for k, itemid in pairs(private.data[MODNAME].wod.id) do
					local fixeditem = strsub(itemid, 1, -1)
					info.text = core:itemString(MODNAME, xp, k, itemid)
					info.func = function() namespace.profiles.Default.tracking[fixeditem] = not namespace.profiles.Default.tracking[fixeditem] core:textUpdate(MODNAME) end
					info.checked = function() return namespace.profiles.Default.tracking[fixeditem] end
					UIDropDownMenu_AddButton(info, level)
				end
			end
			if UIDROPDOWNMENU_MENU_VALUE == "submenu7" then
				info.keepShownOnClick = 1
				xp = "leg"
				for k, itemid in pairs(private.data[MODNAME].leg.id) do
					local fixeditem = strsub(itemid, 1, -1)
					info.text = core:itemString(MODNAME, xp, k, itemid)
					info.func = function() namespace.profiles.Default.tracking[fixeditem] = not namespace.profiles.Default.tracking[fixeditem] core:textUpdate(MODNAME) end
					info.checked = function() return namespace.profiles.Default.tracking[fixeditem] end
					UIDropDownMenu_AddButton(info, level)
				end
			end
		end
	end
end
